/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            luascript.h
 *
 *  Tue Jan 10 14:43:39 CET 2012
 *  Copyright 2012 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of Pracro.
 *
 *  Pracro is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Pracro is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Pracro; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#pragma once

#include <lua.hpp>
#include <lauxlib.h>

#include <string>
#include <map>
#include <vector>

#include "outputwindow.h"
#include "soundplayer.h"

#include <QThread>

class LUAScript
	: public QThread
{
	Q_OBJECT
public:
	class Exception
	{
	public:
		Exception(std::string m) : msg(m) {}
		std::string msg;
	};

	LUAScript(OutputWindow& out);
	~LUAScript();

	void init();
	void cleanup();

	void run();
	void runScript();
	void setScriptFile(QString file);

	OutputWindow& out;

	void lineChange(int lineno);

	void playSound(QString file);
	void sleep(int n);

	volatile bool lua_stop;
	volatile bool lua_stopped;
	QString file;

public slots:
	void stopScript();

signals:
	void reset();
	void lineChanged(int lineno);
	void errorLine(QString file, int lineno, QString error);

protected:
	lua_State *L;

private:
	std::vector<std::pair<std::string, std::string> > scripts;
	std::map<std::string, std::string> values;

	int top;

	bool running;

	SoundPlayer player;
};
